@@ -28,6 +28,20 @@ log = logging.getLogger(__name__)
 # fix for a very specific upstream issue.
 # Related: https://github.com/PyCQA/pylint/issues/3518
 os.environ['PYGAME_HIDE_SUPPORT_PROMPT'] = 'hide'
+DEPRECATION_CODES = {
+    'W0402',  # Uses of a deprecated module %r
+    'W1505',  # Using deprecated method %s()
+    'W1511',  # Using deprecated argument %s of method %s()
+    'W1512',  # Using deprecated class %s of module %s
+    'W1513',  # Using deprecated decorator %s()
+}
+UNNECESSITY_CODES = {
+    'W0611',  # Unused import %s
+    'W0612',  # Unused variable %r
+    'W0613',  # Unused argument %r
+    'W0614',  # Unused import %s from wildcard import
+    'W1304',  # Unused-format-string-argument
+}
 
 
 class PylintLinter:
@@ -146,13 +160,22 @@ class PylintLinter:
             elif diag['type'] == 'warning':
                 severity = lsp.DiagnosticSeverity.Warning
 
-            diagnostics.append({
+            code = diag['message-id']
+
+            diagnostic = {
                 'source': 'pylint',
                 'range': err_range,
                 'message': '[{}] {}'.format(diag['symbol'], diag['message']),
                 'severity': severity,
-                'code': diag['message-id']
-            })
+                'code': code
+            }
+
+            if code in UNNECESSITY_CODES:
+                diagnostic['tags'] = [lsp.DiagnosticTag.Unnecessary]
+            if code in DEPRECATION_CODES:
+                diagnostic['tags'] = [lsp.DiagnosticTag.Deprecated]
+
+            diagnostics.append(diagnostic)
         cls.last_diags[document.path] = diagnostics
         return diagnostics
 
@@ -295,24 +318,27 @@ def _parse_pylint_stdio_result(document, stdout):
             'W': lsp.DiagnosticSeverity.Warning,
         }
         severity = severity_map[code[0]]
-        diagnostics.append(
-            {
-                'source': 'pylint',
-                'code': code,
-                'range': {
-                    'start': {
-                        'line': line,
-                        'character': character
-                    },
-                    'end': {
-                        'line': line,
-                        # no way to determine the column
-                        'character': len(document.lines[line]) - 1
-                    }
+        diagnostic = {
+            'source': 'pylint',
+            'code': code,
+            'range': {
+                'start': {
+                    'line': line,
+                    'character': character
                 },
-                'message': msg,
-                'severity': severity,
-            }
-        )
+                'end': {
+                    'line': line,
+                    # no way to determine the column
+                    'character': len(document.lines[line]) - 1
+                }
+            },
+            'message': msg,
+            'severity': severity,
+        }
+        if code in UNNECESSITY_CODES:
+            diagnostic['tags'] = [lsp.DiagnosticTag.Unnecessary]
+        if code in DEPRECATION_CODES:
+            diagnostic['tags'] = [lsp.DiagnosticTag.Deprecated]
+        diagnostics.append(diagnostic)
 
     return diagnostics
